#include <unistd.h>
#include <sys/types.h>
#include <time.h>
#include <regex.h>
#include "config.h"

#define LINE 1024
#define DEBUG 0

#define OAK_VERSION 1.3.8
#define OAK_VERSION_STRING "1.3.8"

typedef struct _oak_list {
  int size;
  int avail;
  void **list;
} oak_list;

typedef struct _oak_fire {
  time_t next;       /* the unix time at which to fire next */
  time_t interval;   /* the number of seconds between intervals */
} oak_fire;


typedef struct _oak_queue {
  char *name;
  char **msgs;
  char **host;
  int *msgcount;
  int nummsgs;
  oak_fire *fire;
  oak_list *actionlist;
  int locktime;
  char *header;
  int trashqueue;
  int prescan;
} oak_queue;

typedef struct _oak_action {
  char *name;
  int argc;
  char **argv;
  int haslimits;
  int limit_lines;
  int limit_linelen;
  int limit_numhosts;
  int limit_hostents;
} oak_action;


typedef oak_list oak_matchlist;
typedef oak_list oak_queuelist;
typedef oak_list oak_timelist;

typedef struct _oak_matchline {
  regex_t regex;
  oak_queuelist ql;
  oak_queuelist locked_queuelist;
  oak_timelist  lock_times;
} oak_matchline;

typedef struct _oak_global {
  char logfile[1024];
  oak_queuelist queuelist;
  oak_matchlist matchlist;
  int nukepid;
  int nukeciscoid;
  int nukesmqid;
  int onlyhosts;
  int ignorehosts;
  int debug;
  oak_list hostlist;
  int hostsort;
  oak_list hostsortlist;
  char replacestr[100];
} oak_global;

oak_global g;

/* -------------------------------- action.c -------------------------------- */
extern oak_action *action_create(char *name, int argc, char **argv);
extern oak_action *action_create_from_argstring(char *name, char *args);
extern void action_set_limits(oak_action *act, int lines, int linelen, int numhosts, int hostents);
extern void action_execute(oak_action *act, oak_queue *q);
extern char *action_get_name(oak_action *act);

/* -------------------------------- fire.c -------------------------------- */
extern oak_fire *fire_create(char *in);
extern int fire_is_time(oak_fire *f);

/* -------------------------------- global.c -------------------------------- */
extern void oak_global_init(oak_global *g);
extern int oak_global_is_debug(oak_global *g);
extern void oak_global_set_debug(oak_global *g);
extern void oak_global_set_nodebug(oak_global *g);
extern char *oak_global_get_logfile(oak_global *g);
extern void oak_global_set_logfile(oak_global *g, char *filename);
extern oak_queuelist *oak_global_get_queuelist(oak_global *g);
extern oak_matchlist *oak_global_get_matchlist(oak_global *g);
extern int oak_global_is_nukepid(oak_global *g);
extern void oak_global_set_nukepid(oak_global *g);
extern void oak_global_set_no_nukepid(oak_global *g);
extern int oak_global_is_nukeciscoid(oak_global *g);
extern void oak_global_set_nukeciscoid(oak_global *g);
extern void oak_global_set_no_nukeciscoid(oak_global *g);
extern int oak_global_is_nukesmqid(oak_global *g);
extern void oak_global_set_nukesmqid(oak_global *g);
extern void oak_global_set_no_nukesmqid(oak_global *g);
extern char *oak_global_get_replacestr(oak_global *g);
extern void oak_global_set_replacestr(oak_global *g, char *in);
extern int oak_global_host_is_ok(oak_global *g, char *host);
extern int oak_global_set_ignorehosts(oak_global *g, char *hosts);
extern int oak_global_set_onlyhosts(oak_global *g, char *hosts);
extern void _oak_global_set_hostlist(oak_global *g, char *hosts);

/* -------------------------------- list.c -------------------------------- */
extern int oak_list_create(oak_list *l);
extern int oak_list_get_size(oak_list *l);
extern void *oak_list_get_element(oak_list *l, int n);
extern int oak_list_append_element(oak_list *l, void *element);
extern int oak_list_remove_element(oak_list *l, int n);
extern void oak_list_free_all(oak_list *l, void (*elefree)(void *));
extern void oak_list_free_simple(oak_list *l);

/* -------------------------------- matchline.c -------------------------------- */
extern oak_matchline *matchline_create(char *regex, char *queuenames);
extern int matchline_set_queuelist_from_string(oak_matchline *m, char *queuenames);
extern int matchline_string_matches(oak_matchline *m, char *instr, char *out);
extern oak_queuelist *matchline_get_queuelist(oak_matchline *m);
extern int matchline_locked_for_queue(oak_matchline *m, oak_queue *q);
extern void matchline_set_lock(oak_matchline *m, oak_queue *q);

/* -------------------------------- oak.c -------------------------------- */
#ifdef HAVE_UNISTD_H
#endif
extern int main(int argc, char **argv);
extern void readconfig(char *file);
extern char *get_line_from_log(char *log);
extern char *get_host_from_log(char *log);
extern void print_usage(int err);

/* -------------------------------- queue.c -------------------------------- */
extern oak_queue *queue_create(char *name, oak_list *actionlist, oak_fire *fire);
extern char *queue_get_name(oak_queue *q);
extern int queue_is_prescan(oak_queue *q);
extern void queue_set_prescan(oak_queue *q);
extern int queue_get_nummsgs(oak_queue *q);
extern char *queue_get_msg_n(oak_queue *q, int n);
extern char *queue_get_host_n(oak_queue *q, int n);
extern int queue_get_msgcount_n(oak_queue *q, int n);
extern void queue_set_header(oak_queue *q, char *header);
extern char *queue_get_header(oak_queue *q);
extern int queue_is_locking(oak_queue *q);
extern void queue_set_locktime_fromstring(oak_queue *q, char *in);
extern int queue_get_locktime(oak_queue *q);
extern void queue_add_msg(oak_queue *q, char *msg, char *host);
extern void queue_delete_msgs(oak_queue *q);
extern oak_fire *queue_get_fire(oak_queue *q);
extern oak_list *queue_get_actionlist(oak_queue *q);
extern void queue_set_fire(oak_queue *q, oak_fire *fire);
extern void queue_set_actionlist(oak_queue *q, oak_list *actionlist);
extern char *queue_to_string(oak_queue *q);
extern char *queue_to_string_truncate(oak_queue *q, int lines, int linelen, int numhosts, int hostents);

/* -------------------------------- queuelist.c -------------------------------- */
extern oak_queue *queuelist_get_queue_by_name(oak_queuelist *ql, char *name);

/* -------------------------------- util.c -------------------------------- */
extern int trim_string_by_lines(char *instr, char *outstr, int lines);
extern void replace_string_sections(char *in, regmatch_t pmatch[], char *out, char *newstr);
extern char **argstring_to_argv(char *in, int *outargc);
